@@ -35,7 +35,6 @@
 #include <gtk/gtk.h>
 
 #include "gpa.h"   
-#include "gpapastrings.h"
 
 #include "gtktools.h"
 #include "gpawidgets.h"
@@ -50,12 +49,10 @@
 #include "gpafileverifyop.h"
 
 
-/* FIXME:  Move to a gloabl file.  */
-#ifndef DIM
-#define DIM(array) (sizeof (array) / sizeof (*array))
+#if ! GTK_CHECK_VERSION (2, 10, 0)
+#define GTK_STOCK_SELECT_ALL "gtk-select-all"
 #endif
 
-
 
 /* Object and class definition.  */
 struct _GpaFileManager
@@ -92,8 +89,8 @@ enum
   FILE_N_COLUMNS
 };
 
+#define DND_TARGET_URI_LIST 1
 
-#define DND_TARGET_URI_LIST 0
 
 /* Drag and drop target list. */
 static GtkTargetEntry dnd_target_list[] = 
@@ -563,6 +560,7 @@ fileman_action_new (GpaFileManager *fileman, GtkWidget **menubar,
     "      <menuitem action='WindowsKeyringEditor'/>"
     "      <menuitem action='WindowsFileManager'/>"
     "      <menuitem action='WindowsClipboard'/>"
+    "      <menuitem action='WindowsCardManager'/>"
     "    </menu>"
     "    <menu action='Help'>"
 #if 0
@@ -584,6 +582,7 @@ fileman_action_new (GpaFileManager *fileman, GtkWidget **menubar,
     "    <separator/>"
     "    <toolitem action='WindowsKeyringEditor'/>"
     "    <toolitem action='WindowsClipboard'/>"
+    "    <toolitem action='WindowsCardManager'/>"
 #if 0
     "    <toolitem action='HelpContents'/>"
 #endif
@@ -597,6 +596,7 @@ fileman_action_new (GpaFileManager *fileman, GtkWidget **menubar,
   GError *error;
 
   action_group = gtk_action_group_new ("MenuActions");
+  gtk_action_group_set_translation_domain (action_group, PACKAGE);
   gtk_action_group_add_actions (action_group, entries, G_N_ELEMENTS (entries),
 				fileman);
   gtk_action_group_add_actions (action_group, gpa_help_menu_action_entries,
@@ -623,6 +623,8 @@ fileman_action_new (GpaFileManager *fileman, GtkWidget **menubar,
   /* Fixup the icon theme labels which are too long for the toolbar.  */
   action = gtk_action_group_get_action (action_group, "WindowsKeyringEditor");
   g_object_set (action, "short_label", _("Keyring"), NULL);
+  action = gtk_action_group_get_action (action_group, "WindowsCardManager");
+  g_object_set (action, "short_label", _("Card"), NULL);
 
   /* Take care of sensitiveness of widgets.  */
   action = gtk_action_group_get_action (action_group, "FileSign");
