@@ -1,6 +1,6 @@
 /* gpa.h  -  main header
    Copyright (C) 2000, 2001 G-N-U GmbH.
-   Copyright (C) 2008 g10 Code GmbH.
+   Copyright (C) 2008, 2009 g10 Code GmbH.
 
    This file is part of GPA.
 
@@ -28,7 +28,6 @@
 #include <string.h>
 #include <assuan.h>
 
-#include "gpadefs.h"
 #include "gpgmetools.h"
 #include "options.h"
 #include "icons.h"
@@ -52,14 +51,17 @@ extern GtkWidget *global_windowTip;
 extern GList *global_defaultRecipients;
 extern gchar *gnupg_homedir;
 extern gboolean cms_hack;
-
+extern gboolean disable_ticker;
 
 /* Show the keyring editor dialog.  */
-void gpa_open_keyring_editor (GtkAction *action, void *data);
+void gpa_open_key_manager (GtkAction *action, void *data);
 
 /* Show the filemanager dialog.  */
 void gpa_open_filemanager (GtkAction *action, void *data);
 
+/* Show the cardmanager dialog.  */
+void gpa_open_cardmanager (GtkAction *action, void *data);
+
 /* Show the filemanager dialog.  */
 void gpa_open_clipboard (GtkAction *action, void *data);
 
@@ -68,12 +70,14 @@ static const GtkActionEntry gpa_windows_menu_action_entries[] =
   {
       { "Windows", NULL, N_("_Windows"), NULL },
 
-      { "WindowsKeyringEditor", GPA_STOCK_KEYRING, NULL, NULL,
-	N_("Open the keyring editor"), G_CALLBACK (gpa_open_keyring_editor) },
+      { "WindowsKeyringEditor", GPA_STOCK_KEYMAN, NULL, NULL,
+	N_("Open the keyring editor"), G_CALLBACK (gpa_open_key_manager) },
       { "WindowsFileManager", GPA_STOCK_FILEMAN, NULL, NULL,
 	N_("Open the file manager"), G_CALLBACK (gpa_open_filemanager) },
       { "WindowsClipboard", GPA_STOCK_CLIPBOARD, NULL, NULL,
-	N_("Open the clipboard"), G_CALLBACK (gpa_open_clipboard) }
+	N_("Open the clipboard"), G_CALLBACK (gpa_open_clipboard) },
+      { "WindowsCardManager", GPA_STOCK_CARDMAN, NULL, NULL,
+	N_("Open the card manager"), G_CALLBACK (gpa_open_cardmanager) }
   };
 
 
@@ -101,6 +105,17 @@ void gpa_run_server_continuation (assuan_context_t ctx, gpg_error_t err);
 void gpa_start_server (void);
 void gpa_stop_server (void);
 
+
+typedef struct gpa_filewatch_id_s *gpa_filewatch_id_t;
+typedef void (*gpa_filewatch_cb_t) 
+     (void *user_data, const char *filename, const char *reason);
+void gpa_init_filewatch (void);
+gpa_filewatch_id_t gpa_add_filewatch (const char *filename,
+                                      const char *maskstring,
+                                      gpa_filewatch_cb_t cb,
+                                      void *cb_data);
+
+
 /*-- utils.c --*/
 /* We are so used to these function thus provide them.  */
 void *xmalloc (size_t n);
@@ -110,6 +125,12 @@ char *xstrdup (const char *str);
 
 int translate_sys2libc_fd (assuan_fd_t fd, int for_write);
 
+char *decode_c_string (const char *src);
+
+char *percent_escape (const char *string,
+                      const char *delimiters, int space2plus);
+size_t percent_unescape (char *string, int plus2space);
+
 
 /*-- Convenience macros. -- */
 #define DIM(v)		     (sizeof(v)/sizeof((v)[0]))
