@@ -1,22 +1,25 @@
 /* gpafileverifyop.c - The GpaOperation object.
- *	Copyright (C) 2003, Miguel Coca.
- *
- * This file is part of GPA
- *
- * GPA is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * GPA is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
- */
+   Copyright (C) 2003 Miguel Coca.
+   Copyright (C) 2008 g10 Code GmbH.
+  
+   This file is part of GPA.
+
+   GPA is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
+
+   GPA is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
+   License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, see <http://www.gnu.org/licenses/>.  */
+
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
 
 #include <glib.h>
 
@@ -73,9 +76,10 @@ gpa_file_verify_operation_init (GpaFileVerifyOperation *op)
 }
 
 static GObject*
-gpa_file_verify_operation_constructor (GType type,
-					guint n_construct_properties,
-					GObjectConstructParam *construct_properties)
+gpa_file_verify_operation_constructor
+(GType type,
+ guint n_construct_properties,
+ GObjectConstructParam *construct_properties)
 {
   GObject *object;
   GpaFileVerifyOperation *op;
@@ -96,6 +100,7 @@ gpa_file_verify_operation_constructor (GType type,
   /* Give a title to the progress dialog */
   gtk_window_set_title (GTK_WINDOW (GPA_FILE_OPERATION (op)->progress_dialog),
 			_("Verifying..."));
+
   /* Create the verification dialog */
   op->dialog = gpa_file_verify_dialog_new (GPA_OPERATION (op)->window);
   g_signal_connect (G_OBJECT (op->dialog), "response",
@@ -104,6 +109,7 @@ gpa_file_verify_operation_constructor (GType type,
   return object;
 }
 
+
 static void
 gpa_file_verify_operation_class_init (GpaFileVerifyOperationClass *klass)
 {
@@ -231,53 +237,81 @@ is_detached_sig (const gchar *filename, gchar **signature_file,
 
 static gboolean
 gpa_file_verify_operation_start (GpaFileVerifyOperation *op,
-				 const gchar *sig_filename)
+				 gpa_file_item_t file_item)
 {
-  gpg_error_t err;
+  gpgme_error_t err;
 
-  if (is_detached_sig (sig_filename, &op->signature_file, &op->signed_file,
-		       GPA_OPERATION (op)->window))
+  if (file_item->direct_in)
     {
-      /* Allocate data objects for a detached signature */
-      op->sig_fd = gpa_open_input (op->signature_file, &op->sig, 
-				   GPA_OPERATION (op)->window);
-      if (op->sig_fd == -1)
+      /* Direct input is always an inline signature.  */
+
+      /* No copy is made.  */
+      err = gpgme_data_new_from_mem (&op->sig, file_item->direct_in,
+				     file_item->direct_in_len, 0);
+      if (err)
 	{
+	  gpa_gpgme_warning (err);
 	  return FALSE;
 	}
-      op->signed_text_fd = gpa_open_input (op->signed_file, &op->signed_text,
-					   GPA_OPERATION (op)->window);
-      if (op->signed_text_fd == -1)
+      
+      err = gpgme_data_new (&op->plain);
+      if (err)
 	{
+	  gpa_gpgme_warning (err);
 	  gpgme_data_release (op->sig);
-	  close (op->sig_fd);
+	  op->sig = NULL;
 	  return FALSE;
 	}
-      op->plain = NULL;
     }
   else
     {
-      /* Allocate data object for non-detached signatures */
-      op->sig_fd = gpa_open_input (sig_filename, &op->sig, 
-				   GPA_OPERATION (op)->window);
-      if (op->sig_fd == -1)
+      const gchar *sig_filename = file_item->filename_in;
+      
+      if (is_detached_sig (sig_filename, &op->signature_file, &op->signed_file,
+			   GPA_OPERATION (op)->window))
 	{
-	  return FALSE;
+	  /* Allocate data objects for a detached signature */
+	  op->sig_fd = gpa_open_input (op->signature_file, &op->sig, 
+				       GPA_OPERATION (op)->window);
+	  if (op->sig_fd == -1)
+	    {
+	      return FALSE;
+	    }
+	  op->signed_text_fd = gpa_open_input (op->signed_file, &op->signed_text,
+					       GPA_OPERATION (op)->window);
+	  if (op->signed_text_fd == -1)
+	    {
+	      gpgme_data_release (op->sig);
+	      close (op->sig_fd);
+	      return FALSE;
+	    }
+	  op->plain = NULL;
 	}
-      if (gpg_err_code (gpgme_data_new (&op->plain)) != GPG_ERR_NO_ERROR)
+      else
 	{
-	  gpgme_data_release (op->sig);
-	  close (op->sig_fd);
-	  return FALSE;
+	  /* Allocate data object for non-detached signatures */
+	  op->sig_fd = gpa_open_input (sig_filename, &op->sig, 
+				       GPA_OPERATION (op)->window);
+	  if (op->sig_fd == -1)
+	    {
+	      return FALSE;
+	    }
+	  err = gpgme_data_new (&op->plain);
+	  if (err)
+	    {
+	      gpgme_data_release (op->sig);
+	      close (op->sig_fd);
+	      return FALSE;
+	    }
+	  op->signed_text_fd = -1;
+	  op->signed_text = NULL;
 	}
-      op->signed_text_fd = -1;
-      op->signed_text = NULL;
     }
 
   /* Start the operation */
   err = gpgme_op_verify_start (GPA_OPERATION (op)->context->ctx, op->sig,
 			       op->signed_text, op->plain);
-  if (gpg_err_code (err) != GPG_ERR_NO_ERROR)
+  if (err)
     {
       gpa_gpgme_warning (err);
       return FALSE;
@@ -286,15 +320,19 @@ gpa_file_verify_operation_start (GpaFileVerifyOperation *op,
   gtk_widget_show_all (GPA_FILE_OPERATION (op)->progress_dialog);
   gpa_progress_dialog_set_label (GPA_PROGRESS_DIALOG 
 				 (GPA_FILE_OPERATION (op)->progress_dialog),
-				 sig_filename);
+				 file_item->direct_name
+				 ? file_item->direct_name
+				 : file_item->filename_in);
+
   return TRUE;
 }
 
+
 static void
 gpa_file_verify_operation_next (GpaFileVerifyOperation *op)
 {
-  if (!GPA_FILE_OPERATION (op)->current ||
-      !gpa_file_verify_operation_start (op, GPA_FILE_OPERATION (op)
+  if (! GPA_FILE_OPERATION (op)->current ||
+      ! gpa_file_verify_operation_start (op, GPA_FILE_OPERATION (op)
 					 ->current->data))
     {
       /* All files have been verified: show the results dialog */
@@ -302,26 +340,54 @@ gpa_file_verify_operation_next (GpaFileVerifyOperation *op)
     }
 }
 
+
 static void
 gpa_file_verify_operation_done_cb (GpaContext *context, 
 				    gpg_error_t err,
 				    GpaFileVerifyOperation *op)
 {
+  gpa_file_item_t file_item = GPA_FILE_OPERATION (op)->current->data;
+
+  if (file_item->direct_in)
+    {
+      size_t len;
+      char *plain_gpgme = gpgme_data_release_and_get_mem (op->plain,
+							   &len);
+      op->plain = NULL;
+      /* Do the memory allocation dance.  */
+
+      if (plain_gpgme)
+	{
+	  /* Conveniently make ASCII stuff into a string.  */
+	  file_item->direct_out = g_malloc (len + 1);
+	  memcpy (file_item->direct_out, plain_gpgme, len);
+	  gpgme_free (plain_gpgme);
+	  file_item->direct_out[len] = '\0';
+	  /* Yep, excluding the trailing zero.  */
+	  file_item->direct_out_len = len;
+	}
+      else
+	{
+	  file_item->direct_out = NULL;
+	  file_item->direct_out_len = 0;
+	}
+    }
+
   /* Do clean up on the operation */
-  if (op->plain) 
-    {
-      gpgme_data_release (op->plain);
-    }
-  if (op->signed_text)
-    {
-      gpgme_data_release (op->signed_text);
-      close (op->signed_text_fd);
-    }
+  gpgme_data_release (op->plain);
+  op->plain = NULL;
+  gpgme_data_release (op->signed_text);
+  op->signed_text = NULL;
+  close (op->signed_text_fd);
+  op->signed_text_fd = -1;
   gpgme_data_release (op->sig);
+  op->sig = NULL;
   close (op->sig_fd);
+  op->sig_fd = -1;
+
   gtk_widget_hide (GPA_FILE_OPERATION (op)->progress_dialog);
   /* Check for error */
-  if (gpg_err_code (err) != GPG_ERR_NO_ERROR)
+  if (err)
     {
       /* Abort further verifications */
     }
@@ -330,9 +396,12 @@ gpa_file_verify_operation_done_cb (GpaContext *context,
       gpgme_verify_result_t result;
       
       result = gpgme_op_verify_result (GPA_OPERATION (op)->context->ctx);
-      /* Add the file to the result dialog */
+      /* Add the file to the result dialog.  FIXME: Maybe we should
+	 use the filename without the directory.  */
       gpa_file_verify_dialog_add_file (GPA_FILE_VERIFY_DIALOG (op->dialog),
-				       GPA_FILE_OPERATION (op)->current->data,
+				       file_item->direct_name
+				       ? file_item->direct_name
+				       : file_item->filename_in,
 				       op->signed_file, op->signature_file,
 				       result->signatures);
       /* If this was a dettached sig, reset the signed file */
@@ -343,6 +412,14 @@ gpa_file_verify_operation_done_cb (GpaContext *context,
 	  g_free (op->signature_file);
 	  op->signature_file = NULL;
 	}
+      else
+	{
+	  /* Otherwise, we created a "file" in direct mode.  */
+	  if (file_item->direct_in)
+	    g_signal_emit_by_name (GPA_OPERATION (op), "created_file",
+				   file_item);
+	}
+
       /* Go to the next file in the list and verify it */
       GPA_FILE_OPERATION (op)->current = g_list_next 
 	(GPA_FILE_OPERATION (op)->current);
@@ -366,13 +443,17 @@ gpa_file_verify_operation_response_cb (GtkDialog *dialog,
 				       gpointer user_data)
 {
   GpaFileVerifyOperation *op = GPA_FILE_VERIFY_OPERATION (user_data);
-  g_signal_emit_by_name (GPA_OPERATION (op), "completed");  
+
+  /* FIXME: Error handling.  */
+  g_signal_emit_by_name (GPA_OPERATION (op), "completed", 0);
 }
 
+
 static void
 gpa_file_verify_operation_done_error_cb (GpaContext *context, gpg_error_t err,
 					 GpaFileVerifyOperation *op)
 {
+  gpa_file_item_t file_item = GPA_FILE_OPERATION (op)->current->data;
   gchar *message;
 
   switch (gpg_err_code (err))
@@ -382,10 +463,14 @@ gpa_file_verify_operation_done_error_cb (GpaContext *context, gpg_error_t err,
       /* Ignore these */
       break;
     case GPG_ERR_NO_DATA:
-      message = g_strdup_printf (_("The file \"%s\" contained no OpenPGP "
-				   "data."),
-				 gpa_file_operation_current_file 
-				 (GPA_FILE_OPERATION(op)));
+      message = g_strdup_printf (file_item->direct_name
+				 ? _("\"%s\" contained no OpenPGP data.")
+				 : _("The file \"%s\" contained no OpenPGP"
+				     "data."),
+				 file_item->direct_name
+				 ? file_item->direct_name
+				 : file_item->filename_in);
+
       gpa_window_error (message, GPA_OPERATION (op)->window);
       g_free (message);
       break;
