@@ -24,11 +24,12 @@
 
 #include "gpa.h"
 #include "keylist.h"
-#include "gpapastrings.h"
 #include "gpawidgets.h"
+#include "convert.h"
 #include "gtktools.h"
 #include "keytable.h"
 #include "icons.h"
+#include "format-dn.h"
 
 
 /* Properties */
@@ -148,6 +149,18 @@ gpa_keylist_set_property (GObject     *object,
     }
 }
 
+
+static void
+gpa_keylist_dispose (GObject *object)
+{  
+  GpaKeyList *list = GPA_KEYLIST (object);
+
+  list->disposed = 1;
+
+  G_OBJECT_CLASS (parent_class)->dispose (object);
+}
+
+
 static void
 gpa_keylist_finalize (GObject *object)
 {  
@@ -156,6 +169,7 @@ gpa_keylist_finalize (GObject *object)
   /* Dereference all keys in the list */
   g_list_foreach (list->keys, (GFunc) gpgme_key_unref, NULL);
   g_list_free (list->keys);
+  list->keys = NULL;
   gpa_gpgme_release_keyarray (list->initial_keys);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -163,28 +177,13 @@ gpa_keylist_finalize (GObject *object)
 
 
 static void
-gpa_keylist_init (GpaKeyList *list)
+gpa_keylist_init (GTypeInstance *instance, void *class_ptr)
 {
-
-}
-
-
-static GObject*
-gpa_keylist_constructor (GType type,
-                         guint n_construct_properties,
-                         GObjectConstructParam *construct_properties)
-{
-  GObject *object;
-  GpaKeyList *list;
+  GpaKeyList *list = GPA_KEYLIST (instance);
   GtkListStore *store;
   GtkTreeSelection *selection; 
 
-  object = parent_class->constructor (type,
-				      n_construct_properties,
-				      construct_properties);
-  list = GPA_KEYLIST (object);
-
-   /* Init the model */
+  /* Setup the model.  */
   store = gtk_list_store_new (GPA_KEYLIST_N_COLUMNS,
 			      G_TYPE_STRING,
 			      G_TYPE_STRING,
@@ -199,13 +198,14 @@ gpa_keylist_constructor (GType type,
 			      G_TYPE_ULONG,
 			      G_TYPE_LONG);
 
-  /* The view */
+  /* Setup the view.  */
   gtk_tree_view_set_model (GTK_TREE_VIEW (list), GTK_TREE_MODEL (store));
   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (list), TRUE);
   gpa_keylist_set_brief (list);
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
   gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
-  /* Load the keyring */
+
+  /* Load the keyring.  */
   add_trustdb_dialog (list);
   if (list->initial_keys)
     {
@@ -226,18 +226,20 @@ gpa_keylist_constructor (GType type,
       gpa_keytable_list_keys (gpa_keytable_get_public_instance(),
                               gpa_keylist_next, gpa_keylist_end, list);
     }
-  return object;
+
 }
 
 
+
 static void
-gpa_keylist_class_init (GpaKeyListClass *klass)
+gpa_keylist_class_init (void *class_ptr, void *class_data)
 {
+  GpaKeyListClass *klass = class_ptr;
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   
   parent_class = g_type_class_peek_parent (klass);
 
-  object_class->constructor = gpa_keylist_constructor;
+  object_class->dispose = gpa_keylist_dispose;
   object_class->finalize = gpa_keylist_finalize;
   object_class->set_property = gpa_keylist_set_property;
   object_class->get_property = gpa_keylist_get_property;
@@ -297,17 +299,17 @@ gpa_keylist_get_type (void)
         sizeof (GpaKeyListClass),
         (GBaseInitFunc) NULL,
         (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc) gpa_keylist_class_init,
+        gpa_keylist_class_init,
         NULL,           /* class_finalize */
         NULL,           /* class_data */
         sizeof (GpaKeyList),
         0,              /* n_preallocs */
-        (GInstanceInitFunc) gpa_keylist_init,
+        gpa_keylist_init,
       };
       
       keylist_type = g_type_register_static (GTK_TYPE_TREE_VIEW,
-						  "GpaKeyList",
-						  &keylist_info, 0);
+                                             "GpaKeyList",
+                                             &keylist_info, 0);
     }
   
   return keylist_type;
@@ -369,9 +371,16 @@ remove_trustdb_dialog (GpaKeyList * keylist)
 static const gchar *
 get_key_pixbuf (gpgme_key_t key)
 {
-  if (gpa_keytable_lookup_key (gpa_keytable_get_secret_instance (),
-			       key->subkeys->fpr) != NULL)
-    return GPA_STOCK_SECRET_KEY;
+  gpgme_key_t seckey;
+
+  seckey = gpa_keytable_lookup_key (gpa_keytable_get_secret_instance (),
+                                    key->subkeys->fpr);
+  if (seckey)
+    {
+      if (seckey->subkeys && seckey->subkeys->is_cardkey)
+        return GPA_STOCK_SECRET_CARDKEY;
+      return GPA_STOCK_SECRET_KEY;
+    }
   else
     return GPA_STOCK_PUBLIC_KEY;
 }
@@ -406,6 +415,9 @@ gpa_keylist_next (gpgme_key_t key, gpointer data)
   /* Remove the dialog if it is being displayed */
   remove_trustdb_dialog (list);
 
+  if (list->disposed)
+    return;  /* Should not access our store anymore.  */
+
   /* Filter out keys we don't want.  */
   if (key && list->protocol != GPGME_PROTOCOL_UNKNOWN
       && key->protocol != list->protocol)
@@ -439,7 +451,10 @@ gpa_keylist_next (gpgme_key_t key, gpointer data)
   expiry = gpa_expiry_date_string (key->subkeys->expires);
   ownertrust = gpa_key_ownertrust_string (key);
   validity = gpa_key_validity_string (key);
-  userid = gpa_gpgme_key_get_userid (key->uids);
+  if (key->protocol == GPGME_PROTOCOL_CMS)
+    userid = gpa_format_dn (key->uids? key->uids->uid : NULL);
+  else
+    userid = gpa_gpgme_key_get_userid (key->uids);
   if (list->public_only)
     has_secret = 0;
   else
