@@ -18,9 +18,7 @@
    along with GPA; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
@@ -115,7 +113,7 @@ gpa_file_verify_dialog_constructor (GType type,
   /* Set up the dialog */
   gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 			  _("_Close"), GTK_RESPONSE_CLOSE, NULL);
-  gtk_window_set_title (GTK_WINDOW (dialog), _("Verify files"));
+  gtk_window_set_title (GTK_WINDOW (dialog), _("Verify documents"));
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
   gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
 
@@ -392,6 +390,15 @@ GtkWidget *gpa_file_verify_dialog_new (GtkWidget *parent)
   return GTK_WIDGET(dialog);
 }
 
+void
+gpa_file_verify_dialog_set_title (GpaFileVerifyDialog *dialog, 
+                                  const char *title)
+{
+  if (dialog && title && *title)
+    gtk_window_set_title (GTK_WINDOW (dialog), title);
+}
+
+
 void gpa_file_verify_dialog_add_file (GpaFileVerifyDialog *dialog,
 				      const gchar *filename,
 				      const gchar *signed_file,
